function N_BODY_PROBLEM( problem_file )

close all;

global m;
global G;
global n;
global d;


problem_name = 'n_body_';
show_error = true;
d=2;
load(problem_file);
n_ts = length(time_steps);
y0 = qv_to_y(q, v);

if( ~exist('ref_traj') || size(ref_traj, 2)~=n_ts )
   fprintf('Recomputing the reference trajectory ...\n');

   ref_traj = integrate_ode('int_reference(', problem_name, time_steps, y0);

   fprintf('saving ...\n');

   save(problem_file, 'ref_traj', '-append');

   fprintf('done\n');
end

if( show_error )
   subplot(1, 2, 1);
end

plot([plot_bounds.MAX_X, plot_bounds.MIN_X], ...
     [plot_bounds.MAX_Y, plot_bounds.MIN_Y], 'k+');

maxm = max(m); maxs = 70;
minm = min(m); mins = 10;
sizes = (m-minm)/(maxm-minm)*(maxs-mins) + mins;

handles = zeros(n, 1);

hold on;

ref_hand = plot_trajectory(ref_traj, plot_bounds, 'k-');

if( show_error )
   subplot(1, 2, 2);
   semilogy(time_steps, ones(size(time_steps))*eps,'k-');
   ylabel('Relativer Fehler');
   xlabel('Zeit');
   hold on;
end

test_methods = { ...
'implRungeKutta(''imp euler'',',    'Impliziter Euler', 'b'; ...
'implRungeKutta(''impl midpoint'',','Implizite Mittelpunktsregel', 'g'; ...
'implRungeKutta(''radau2a'',',      'Radau2a',          'r'; ...
'', '', ''};

num_meth = size(test_methods, 1)-1;

legend_handles = zeros(1, num_meth);

best_meth = 0;
best_err = inf;
best_traj = [];

ref_norm = norm(ref_traj);

for ind_m = 1:num_meth
   comp_traj = integrate_ode( test_methods{ind_m, 1}, ...
                              problem_name, time_steps, y0);

   errors = sqrt( sum((comp_traj-ref_traj).^2, 1) )/ref_norm;

   norm_errors = norm(errors);
   if( norm_errors < best_err )
      best_err = norm_errors;
      best_meth = ind_m;
      best_traj = comp_traj;
   end

   if( show_error )
      subplot(1, 2, 2);
      semilogy(time_steps, errors, test_methods{ind_m, 3});

      subplot(1, 2, 1);
   end
   legend_handles(ind_m) = plot_trajectory(comp_traj, ...
                           plot_bounds, [test_methods{ind_m, 3}, '--']);
end

leg=legend([ref_hand, legend_handles], 'Referenz', test_methods{1:ind_m, 2});
set(leg,'fontsize',6);

if( is_matlab() )

   for i=1:n
      handles(i) = plot(q{i}(1), q{i}(2), 'b.', ...
                        'MarkerSize', round(sizes(i)), ...
                        'LineWidth', 1);
      set(handles(i), 'XDataSource', ...
            'max(plot_bounds.MIN_X, min(plot_bounds.MAX_X, q{i}(1, t_ind)))');
      set(handles(i), 'YDataSource', ...
            'max(plot_bounds.MIN_Y, min(plot_bounds.MAX_Y, q{i}(2, t_ind)))');
   end

   [q, v] = y_to_qv(best_traj);

   for t_ind = 2:n_ts
      t = time_steps(t_ind);

      title(['Time : ',num2str(t)]);

      for i=1:n
         refreshdata(handles(i), 'caller');
      end
      drawnow;
   end

else
   [q, v] = y_to_qv(best_traj(:, n_ts));

   for i=1:n
      val_x = max(plot_bounds.MIN_Y, min(plot_bounds.MAX_Y, q{i}(1)));
      val_y = max(plot_bounds.MIN_Y, min(plot_bounds.MAX_Y, q{i}(2)));

      plot(val_x, val_y, 'b.',  ...
                        'MarkerSize', round(sizes(i)), ...
                        'LineWidth', 1);
   end

end


if( show_error && is_matlab() )
   subplot(1, 2, 1);
   hold off;
   subplot(1, 2, 2);
   hold off;
else
   hold off;
end

function hand = plot_trajectory(traj, plot_bounds, varargin)

[q, v] = y_to_qv(traj);

for i=1:length(q)
   hand = plot( max(plot_bounds.MIN_X, min(plot_bounds.MAX_X, q{i}(1, :))), ...
                max(plot_bounds.MIN_Y, min(plot_bounds.MAX_Y, q{i}(2, :))), ...
                varargin{:} );
end


function y = qv_to_y(q, v)

global n;
global d;

vecs = size(q{1}, 2);

y=zeros(2*n*d, vecs);

for i=1:n
   y( (d*(  i-1)+1):(d*  i  ), :) = q{i};
   y( (d*(n+i-1)+1):(d*(n+i)), :) = v{i};
end

function [q, v] = y_to_qv(y)

global n;
global d;

q = cell(n, 1);
v = cell(n, 1);

for i=1:n
   q{i} = y( d*(  i-1)+1: d*  i  , :);
   v{i} = y( d*(n+i-1)+1: d*(n+i), :);
end